@@ -1,12 +1,14 @@
 context("fprod")
 
+if(!is.null(attributes(identical(FALSE, TRUE)))) stop("OECD label issue")
+
 bprod <- base::prod
 
 # rm(list = ls())
 set.seed(101)
 x <- rnorm(100)
-w <- abs(100*rnorm(100))
-wdat <- abs(100*rnorm(32))
+w <- abs(5*rnorm(100))
+wdat <- abs(5*rnorm(32))
 xNA <- x
 wNA <- w
 wdatNA <- wdat
@@ -38,12 +40,6 @@ wprod <- function(x, w, na.rm = FALSE) {
   bprod(x*w)
 }
 
-wBY <- function(x, f, FUN, w, ...) {
-  if(is.atomic(x) && !is.array(x)) return(mapply(FUN, split(x, f), split(w, f), ...))
-  wspl <- split(w, f)
-  if(is.atomic(x)) return(dapply(x, function(xi) mapply(FUN, split(xi, f), wspl, ...)))
-  qDF(dapply(x, function(xi) mapply(FUN, split(xi, f), wspl, ...), return = "matrix"))
-}
 
 
 test_that("fprod performs like base::prod", {
@@ -105,18 +101,18 @@ test_that("fprod with weights performs like wprod (defined above)", {
   expect_equal(fprod(mtcars, w = wdat, na.rm = FALSE), dapply(mtcars, wprod, wdat))
   expect_equal(fprod(mtcNA, w = wdat, na.rm = FALSE), dapply(mtcNA, wprod, wdat))
   expect_equal(fprod(mtcNA, w = wdat), dapply(mtcNA, wprod, wdat, na.rm = TRUE))
-  expect_equal(fprod(x, f, w), wBY(x, f, wprod, w))
-  expect_equal(fprod(x, f, w, na.rm = FALSE), wBY(x, f, wprod, w))
-  expect_equal(fprod(xNA, f, w, na.rm = FALSE), wBY(xNA, f, wprod, w))
-  expect_equal(fprod(xNA, f, w), wBY(xNA, f, wprod, w, na.rm = TRUE))
-  expect_equal(fprod(m, g, wdat), wBY(m, gf, wprod, wdat))
-  expect_equal(fprod(m, g, wdat, na.rm = FALSE), wBY(m, gf, wprod, wdat))
-  expect_equal(fprod(mNA, g, wdat, na.rm = FALSE),  wBY(mNA, gf, wprod, wdat))
-  expect_equal(fprod(mNA, g, wdat), wBY(mNA, gf, wprod, wdat, na.rm = TRUE))
-  expect_equal(fprod(mtcars, g, wdat), wBY(mtcars, gf, wprod, wdat))
-  expect_equal(fprod(mtcars, g, wdat, na.rm = FALSE), wBY(mtcars, gf, wprod, wdat))
-  expect_equal(fprod(mtcNA, g, wdat, na.rm = FALSE), wBY(mtcNA, gf, wprod, wdat))
-  expect_equal(fprod(mtcNA, g, wdat), wBY(mtcNA, gf, wprod, wdat, na.rm = TRUE))
+  expect_equal(fprod(x, f, w), BY(x, f, wprod, w))
+  expect_equal(fprod(x, f, w, na.rm = FALSE), BY(x, f, wprod, w))
+  expect_equal(fprod(xNA, f, w, na.rm = FALSE), BY(xNA, f, wprod, w))
+  expect_equal(fprod(xNA, f, w), BY(xNA, f, wprod, w, na.rm = TRUE))
+  expect_equal(fprod(m, g, wdat), BY(m, gf, wprod, wdat))
+  expect_equal(fprod(m, g, wdat, na.rm = FALSE), BY(m, gf, wprod, wdat))
+  expect_equal(fprod(mNA, g, wdat, na.rm = FALSE),  BY(mNA, gf, wprod, wdat))
+  expect_equal(fprod(mNA, g, wdat), BY(mNA, gf, wprod, wdat, na.rm = TRUE))
+  expect_equal(fprod(mtcars, g, wdat), BY(mtcars, gf, wprod, wdat))
+  expect_equal(fprod(mtcars, g, wdat, na.rm = FALSE), BY(mtcars, gf, wprod, wdat))
+  expect_equal(fprod(mtcNA, g, wdat, na.rm = FALSE), BY(mtcNA, gf, wprod, wdat))
+  expect_equal(fprod(mtcNA, g, wdat), BY(mtcNA, gf, wprod, wdat, na.rm = TRUE))
   # missing weights
   expect_equal(fprod(NA, w = NA), wprod(NA, NA))
   expect_equal(fprod(NA, w = NA, na.rm = FALSE), wprod(NA, NA))
@@ -139,18 +135,18 @@ test_that("fprod with weights performs like wprod (defined above)", {
   expect_equal(fprod(mtcars, w = wdatNA, na.rm = FALSE), dapply(mtcars, wprod, wdatNA))
   expect_equal(fprod(mtcNA, w = wdatNA, na.rm = FALSE), dapply(mtcNA, wprod, wdatNA))
   expect_equal(fprod(mtcNA, w = wdatNA), dapply(mtcNA, wprod, wdatNA, na.rm = TRUE))
-  expect_equal(fprod(x, f, wNA), wBY(x, f, wprod, wNA, na.rm = TRUE))
-  expect_equal(fprod(x, f, wNA, na.rm = FALSE), wBY(x, f, wprod, wNA))
-  expect_equal(fprod(xNA, f, wNA, na.rm = FALSE), wBY(xNA, f, wprod, wNA))
-  expect_equal(fprod(xNA, f, wNA), wBY(xNA, f, wprod, wNA, na.rm = TRUE))
-  expect_equal(fprod(m, g, wdatNA), wBY(m, gf, wprod, wdatNA, na.rm = TRUE))
-  expect_equal(fprod(m, g, wdatNA, na.rm = FALSE), wBY(m, gf, wprod, wdatNA))
-  expect_equal(fprod(mNA, g, wdatNA, na.rm = FALSE),  wBY(mNA, gf, wprod, wdatNA))
-  expect_equal(fprod(mNA, g, wdatNA), wBY(mNA, gf, wprod, wdatNA, na.rm = TRUE))
-  expect_equal(fprod(mtcars, g, wdatNA), wBY(mtcars, gf, wprod, wdatNA, na.rm = TRUE))
-  expect_equal(fprod(mtcars, g, wdatNA, na.rm = FALSE), wBY(mtcars, gf, wprod, wdatNA))
-  expect_equal(fprod(mtcNA, g, wdatNA, na.rm = FALSE), wBY(mtcNA, gf, wprod, wdatNA))
-  expect_equal(fprod(mtcNA, g, wdatNA), wBY(mtcNA, gf, wprod, wdatNA, na.rm = TRUE))
+  expect_equal(fprod(x, f, wNA), BY(x, f, wprod, wNA, na.rm = TRUE))
+  expect_equal(fprod(x, f, wNA, na.rm = FALSE), BY(x, f, wprod, wNA))
+  expect_equal(fprod(xNA, f, wNA, na.rm = FALSE), BY(xNA, f, wprod, wNA))
+  expect_equal(fprod(xNA, f, wNA), BY(xNA, f, wprod, wNA, na.rm = TRUE))
+  expect_equal(fprod(m, g, wdatNA), BY(m, gf, wprod, wdatNA, na.rm = TRUE))
+  expect_equal(fprod(m, g, wdatNA, na.rm = FALSE), BY(m, gf, wprod, wdatNA))
+  expect_equal(fprod(mNA, g, wdatNA, na.rm = FALSE),  BY(mNA, gf, wprod, wdatNA))
+  expect_equal(fprod(mNA, g, wdatNA), BY(mNA, gf, wprod, wdatNA, na.rm = TRUE))
+  expect_equal(fprod(mtcars, g, wdatNA), BY(mtcars, gf, wprod, wdatNA, na.rm = TRUE))
+  expect_equal(fprod(mtcars, g, wdatNA, na.rm = FALSE), BY(mtcars, gf, wprod, wdatNA))
+  expect_equal(fprod(mtcNA, g, wdatNA, na.rm = FALSE), BY(mtcNA, gf, wprod, wdatNA))
+  expect_equal(fprod(mtcNA, g, wdatNA), BY(mtcNA, gf, wprod, wdatNA, na.rm = TRUE))
 })
 
 test_that("fprod performs numerically stable", {
